/**
 * EpicMusicPlayer Playlistgenerator
 *
 * @author yess
 */

package app;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * 
 */
public class Config {

    private File musicdir;
    private String filename;
    private File playlistfile;
    private File wowDirectory;
    private String loadfrom = "";
    private String saveto = "";
    private boolean duplicate = false;
    private boolean saved = true;
    private static String version = "0.6";
   
    public Config() {
        filename = "emp.config";
    }

    
    public void load() throws Exception {
                
        //try load saved config
        try {
        File conffile = new File(filename);
        String a = conffile.getAbsolutePath();
        BufferedReader in = new BufferedReader(new FileReader(filename));
        String line = in.readLine();
        
        while (line != null) {
            if (line.contains("musicfolder")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setMusicFloder(new File(path[1]));
            }
            
            if (line.contains("wowfolder")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setWoWDir(new File(path[1]));
            }
            if (line.contains("loadfrom")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setLoadfrom(path[1]);
            }
            if (line.contains("saveto")) {
                String[] path = line.split("=");
                //System.out.println(path[1]);
                setSaveto(path[1]);
            }
            if (line.contains("dublicate")) {
                String[] dub = line.split("=");
                //System.out.println(dub[1]);
                if(dub[1].equalsIgnoreCase("true"))
                {
                    setDuplicate(true);
                }
            }
            line = in.readLine();
            
        }
        } catch (FileNotFoundException e1){
            e1.printStackTrace();
            throw new Exception("Configuration file not found.");  
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("Error loading config.");  
        }
        
        
        
    }

    public void save() throws IOException {
        BufferedWriter out = new BufferedWriter(new FileWriter(filename, false));
        out.write("musicfolder=" + this.musicdir);
        out.newLine();
        out.write("wowfolder=" + getWoWDir());
        out.newLine();
        out.write("loadfrom=" + getLoadfrom());
        out.newLine();
        out.write("saveto=" + getSaveto());
        out.newLine();
        out.write("dublicate=" + isDuplicate());
        out.newLine();
        out.close();
    }

    /**
     * Set music folder without check if in wow folder
     * @param file
     */
    private void setMusicFloder(File file) {
        if(file == null)
            throw new IllegalArgumentException("MusicFloder not set.");
        if(file.getPath().equals("") || file.getPath().equalsIgnoreCase("null"))
            throw new IllegalArgumentException("MusicFloder not set.");
        
        this.musicdir = file;
    }

    private void setWoWDir(File file)
    {
        if(file == null)
            throw new IllegalArgumentException("WoW folder not set.");
        if(file.getPath().equals("") || file.getPath().equalsIgnoreCase("null"))
            throw new IllegalArgumentException("WoW folder not set.");
        
        wowDirectory = file;
    }
    
    public File getWoWDir()
    {
        return wowDirectory;
    }
    
    public void setMusicDir(File musicdir) throws FileNotFoundException {
        if (!musicdir.exists()) {
            throw new FileNotFoundException("Directory not found.\n" + musicdir.getPath());
        }

        //check if in wow folder
        File addondir = findWowDir(musicdir);
        if(addondir==null){
            throw new FileNotFoundException("Music folder is not in World of Warcraft folder");
        }
        else
        {
            //set music folder
            System.out.println("wowdir="+addondir.getPath());
            this.musicdir = musicdir;
            this.wowDirectory = addondir;
        }   
        
    }
    
    private File findWowDir(File musicdir)
    {
        
        while(musicdir !=null)
        {    
            System.out.println("cheking="+musicdir.getPath());
            String[] children = musicdir.list();
            for(int i=0; i<children.length;i++)
            {
                if(children[i].equalsIgnoreCase("WTF"))
                {
                    return musicdir; //wow folder found
                }
            }
            musicdir = musicdir.getParentFile();
        }
        return null;
    }
    
    public File getPlaylistFile(String accountname){
        return new File(getWoWDir().getPath() + "/WTF/Account/" + accountname + "/SavedVariables/EpicMusicPlayer.lua");
    }

    public List getAccunts(File wowdir) throws FileNotFoundException {

        File accountsdir = new File(wowdir.getPath() + "/WTF/Account/");

        if (!accountsdir.exists()) {
            throw new FileNotFoundException("Account directory not found in: \n" + accountsdir.getPath());
        }

        ArrayList list = new ArrayList();
        String[] children = accountsdir.list();
        for (int i = 0; i < children.length; i++) {
            //System.out.println(children[i]);
            list.add(children[i]);
        }


        return list;
    }

    public File getMusicDir() {
        return musicdir;
    }

    public boolean isSaved() {
        return saved;
    }

    public void setSaved(boolean saved) {
        this.saved = saved;
    }

    public static String getVersion() {
        return version;
    }

    public String getLoadfrom() {
        return loadfrom;
    }

    public void setLoadfrom(String loadfrom) {
        if(loadfrom == null)
            throw new IllegalArgumentException("Account not set.");
        if(loadfrom.equals("") || loadfrom.equalsIgnoreCase("null"))
            throw new IllegalArgumentException("Account not set.");
        
        this.loadfrom = loadfrom;
    }

    public String getSaveto() {
        return saveto;
    }

    public void setSaveto(String saveto) {
        if(saveto == null)
            throw new IllegalArgumentException("Account not set.");
        if(saveto.equals("") || saveto.equalsIgnoreCase("null"))
            throw new IllegalArgumentException("Account not set.");
        
        this.saveto = saveto;
    }

    public boolean isDuplicate() {
        return duplicate;
    }

    public void setDuplicate(boolean duplicate) {
        this.duplicate = duplicate;
    }

}
