/**
 * EpicMusicPlayer Playlistgenerator
 *
 * @author yess
 */

package playlist;

import app.Config;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JTextArea;
import javazoom.jl.decoder.Bitstream;
import javazoom.jl.decoder.BitstreamException;
import javazoom.jl.decoder.Header;

import org.tritonus.share.sampled.file.TAudioFileFormat;

import org.cmc.music.metadata.MusicMetadata;
import org.cmc.music.metadata.MusicMetadataSet;
import org.cmc.music.myid3.MyID3;
import org.cmc.shared.util.Debug;



public class Playlists {

    private List newPlaylist;
    private List<ArrayList> savedPlaylists;
    private List tempCopy;
    private List savedConfig;
    //private ArrayList<String> SummListErrors; //just for summery output
    //private ArrayList<String> SummListAlreadyInList; //just for summery output
    
    private Config conf;
    private JTextArea jLog;
    
    public Playlists(Config conf, JTextArea jlog) {
        
        this.conf = conf;
        this.jLog = jlog;
        //this.jPane = jpane;
        savedConfig = new ArrayList();
        tempCopy = new ArrayList();
        savedPlaylists = new ArrayList();
        
        //chek if there is a playlist
        if(conf.getPlaylistFile(conf.getLoadfrom()).exists())
        {
            try {
                savedPlaylists = readPlaylist(conf.getPlaylistFile(conf.getLoadfrom()).getPath());
            } catch (FileNotFoundException ex) {
                log("Error loading plylist: "+conf.getPlaylistFile(conf.getLoadfrom()).getPath()+" "+ex.getMessage(),0);
                Logger.getLogger(Playlists.class.getName()).log(Level.SEVERE, null, ex);
            }
            
        }
        else //playlist not found maybe running fist time 
        {
            //load own list from jar file
            InputStream is = Playlists.class.getResourceAsStream("/gui/resources/NewList.lua");
            savedPlaylists = readPlaylist(is);
        }
        
        if(savedPlaylists==null) //no saved list or error add defasult list
            {
            /*
            savedPlaylists = new ArrayList();
            Song mp3 = new Song("", "Default", "", "", 0);
            ArrayList list = new ArrayList();                    
            list.add(mp3);
            savedPlaylists.add(list);
            */
            log("Error loading plylist from: "+conf.getPlaylistFile(conf.getLoadfrom()).getPath(),0);
            log("Loading default list.",0);
            InputStream is = Playlists.class.getResourceAsStream("/gui/resources/NewList.lua");
            savedPlaylists = readPlaylist(is);
        }
        
        try{
            readSavedVariables(conf.getPlaylistFile(conf.getLoadfrom()).getPath());            
        }catch(Exception e){
            //load own list from jar file
            InputStream is = Playlists.class.getResourceAsStream("/gui/resources/NewList.lua");
            savedPlaylists = readPlaylist(is);
            Logger.getLogger(Playlists.class.getName()).log(Level.SEVERE, null, e);
        }
    }

    public void expoertSimpleMp3(String filename) {
        filename = filename+"/Playlist.lua";
        List list = savedPlaylists;
        
        try {
            System.out.println("writing playlist to="+filename);
            
            //BufferedWriter out = new BufferedWriter(new FileWriter(filename, true));
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"UTF8"));
            
            out.newLine();
            out.write("PlayList = {");
            out.newLine();

            //write playlists
            for (int listindex = 0; listindex < list.size(); listindex++) {
                
                
                ArrayList mp3list;
                mp3list = (ArrayList) (list.get(listindex));
                
                for (int mp3index = 1; mp3index < mp3list.size(); mp3index++) {    
                    out.write("    {");
                    out.newLine();
                    out.write("      [\"Album\"] = \"" + ((Song) mp3list.get(mp3index)).getSongAlbum() + "\",");
                    out.newLine();
                    out.write("      [\"Song\"] = \"" + ((Song) mp3list.get(mp3index)).getMp3Title() + "\",");
                    out.newLine();
                    out.write("      [\"Name\"] = \"" + ((Song) mp3list.get(mp3index)).getsongPath() + "\",");
                    out.newLine();
                    out.write("      [\"Length\"] = " + ((Song) mp3list.get(mp3index)).getMp3Seconds() + ",");
                    out.newLine();
                    out.write("      [\"Artist\"] =  \"" + ((Song) mp3list.get(mp3index)).getSongArtist() + "\",");
                    out.newLine();
                    //["Genre"] = "Soundtrack",
                    out.write("    }, -- [" + (mp3index + 1) + "]");
                    out.newLine();
                }
                
            }

            out.write("}");
            out.newLine();
            out.close();
        } catch (IOException e) {
        }
    }


    public String getPlaylistName(int index) {
        String title = "";
        
        try {
            ArrayList<Song> l = savedPlaylists.get(index);
            Song song = l.get(0);
            title =  song.getMp3Title();
        } catch (Exception e) {
            Logger.getLogger(Playlists.class.getName()).log(Level.SEVERE, null, e);
        }
        return title;
    }
    
    public ArrayList<Song> getPlaylist(int index) {        
        ArrayList<Song> list = null;
        try {
            list = savedPlaylists.get(index);
        } catch (Exception e) {
            Logger.getLogger(Playlists.class.getName()).log(Level.SEVERE, null, e);
        }
        return list; 
    }
    
    
    public int getPlaylistCount()
    {
        return savedPlaylists.size();
    }
    
    public void removePlaylist(int index) throws Exception {
        this.conf.setSaved(false);
        if(index < savedPlaylists.size()){
            savedPlaylists.remove(index);
        }
        else
        {
            throw new Exception("Index out of range.");
        }
    }
    
    
    public void renamePlaylist(int index, String newname) throws Exception {
        this.conf.setSaved(false);
        if(index < savedPlaylists.size()){
            ArrayList l = (ArrayList) savedPlaylists.get(index);
            Song song = (Song) l.get(0);
            song.setMp3Title(newname);
            l.set(0, song);
        }
        else
        {
            throw new Exception("Index out of range.");
        }
    }

    /*
     * save the all playlists and the savedConfig to savedVariablesPath
     */
    public void save(String account) {
        String musicdir = conf.getMusicDir().getPath();
        //remove wow folder prefix from music dir
        musicdir = musicdir.substring(conf.getWoWDir().getPath().length()+1);
        
        musicdir = musicdir.replaceAll("\\\\", "\\\\\\\\"); 
        musicdir = musicdir+"\\\\";
        //System.out.println("wowdir="+conf.getWoWDir().getPath()+"lenght="+conf.getWoWDir().getPath().length());
        //System.out.println("musicdir="+musicdir);
 
        saveBackup(musicdir);
              
        if(account.equalsIgnoreCase("all-accounts")){
            try {
                List accounts = conf.getAccunts(conf.getWoWDir());
                for (int i = 0; i < accounts.size(); i++) {
                    // deleate the old saved list
                    File f = conf.getPlaylistFile((String)accounts.get(i));
                    f.delete();
                    writePlaylists(savedPlaylists,conf.getPlaylistFile((String)accounts.get(i)).getPath(),musicdir);
                    writeList(savedConfig,conf.getPlaylistFile((String)accounts.get(i)).getPath()); 
                }
                this.conf.setSaved(true);
            } catch (FileNotFoundException ex) {
                Logger.getLogger(Playlists.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        else
        {
            // deleate the old saved list
            File f = conf.getPlaylistFile(account);
            f.delete();
            writePlaylists(savedPlaylists,conf.getPlaylistFile(account).getPath(),musicdir);
            writeList(savedConfig,conf.getPlaylistFile(account).getPath());
            this.conf.setSaved(true);
        }
    }
    
    
    public void saveBackup(String musicdir)
    {
        File f = new File("backup.lua");
        f.delete();
        writePlaylists(savedPlaylists,f.getPath(),musicdir);
        writeList(savedConfig,f.getPath());
        
    }
    
    
    /**
     * calls process on all mp3 files in given directory
     * add all found mp3 files to given list.
     *
     * @param dir
     * @param mp3list
     * @return number of songs added
     */
    public int visitAllFiles(File dir, List mp3list, List sumlist) {
        this.conf.setSaved(false);
        int count = 0;
        
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i = 0; i < children.length; i++) {
                visitAllFiles(new File(dir, children[i]), mp3list, sumlist);
                //System.out.println(dir.getName());
            }
        } else {
            if (dir.getName().toLowerCase().endsWith(".mp3")) {
                try {

                    Song song = processMp3(dir, sumlist);
                    if (song != null) {
                        mp3list.add(song);
                        count++;
                        log("Added " + count + ": \"" + song.getsongPath() + "\"", 1);
                    }
                } catch (Exception ex) {
                    log("oO Error parsing:"+dir.getPath()+"("+ex+")",0);
                    Logger.getLogger(Playlists.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            else if(dir.getName().toLowerCase().endsWith(".ogg")){
                try {

                    Song song = processOgg(dir, sumlist);
                    if (song != null) {
                        mp3list.add(song);
                        count++;
                        log("Added " + count + ": \"" + song.getsongPath() + "\"", 1);
                    }
                } catch (Exception ex) {
                    log("oO Error parsing:"+dir.getPath()+"("+ex+")",0);
                    Logger.getLogger(Playlists.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        return count;
    }

    /**
     * Get mp3tags from file
     * 
     * @param file
     * @return Song
     */
    public Song processMp3(File src, List summlist) throws BitstreamException, IOException {

        Song song = null;
        
        String path = src.getPath();
        //remove addon dir prefix from file path
        path = path.substring(conf.getMusicDir().getPath().length() + 1);
        //replace all \ with \\
        path = path.replaceAll("\\\\", "\\\\\\\\"); 
        
       
        
        int ret = checkInPlaylist(path);
        if(ret > -1 && conf.isDuplicate() == false)
        {
            //System.out.println(file.getName());
            summlist.add(path);
            log("Song already in Playlist: "+getPlaylistName(ret)+" File: "+path,2);
            //throw new FileAlreadyInListException("File already in some Playlist.");
            return null;
        }

            String artist = "";
            String album = "";
            String song_title = "";
           
            
            MusicMetadataSet src_set = new MyID3().read(src); 
            if (src_set == null) // perhaps no metadata
            {
                Debug.debug("src_set", src_set); // dump all info.
            }
            else
            {
                MusicMetadata metadata = (MusicMetadata) src_set.getSimplified();
                artist = metadata.getArtist();
                album = metadata.getAlbum();
                song_title = metadata.getSongTitle();
               
            }
            
            if(artist==null)
            {
                artist="";
            }
            
            if(album==null || album.equals(""))
            {
                album=toAlbum(path);
            }
            if(song_title==null || song_title.equals(""))
            {
                song_title=toTitel(path);
            }
            
            
            //album = "ingame";
            path = path.replace('\"', ' ');
            song_title = song_title.replace('\"', ' ');
            artist = artist.replace('\"', ' ');
            album = album.replace('\"', ' ');
            
            song = new Song(path, song_title, artist, album, getSeconds(src));
   
       
        return song;
    }
    

     /**
     * Get metadata from ogg file
     * 
     * @param file
     * @return Song
     */
    public Song processOgg(File src, List summlist) throws BitstreamException, IOException {

        Song song = null;
        
        String path = src.getPath();
        //remove addon dir prefix from file path
        path = path.substring(conf.getMusicDir().getPath().length() + 1);
        //replace all \ with \\
        path = path.replaceAll("\\\\", "\\\\\\\\"); 
        
       
        
        int ret = checkInPlaylist(path);
        if(ret > -1 && conf.isDuplicate() == false)
        {
            //System.out.println(file.getName());
            summlist.add(path);
            log("Song already in Playlist: "+getPlaylistName(ret)+" File: "+path,2);
            //throw new FileAlreadyInListException("File already in some Playlist.");
            return null;
        }

            String artist = null;
            String album = null;
            String song_title = null;
            int seconds = 0;
            
            /**
             * read meatdata from oog
             */
            
            AudioFileFormat baseFileFormat = null;
            AudioFormat baseFormat = null;
            
            try{
                baseFileFormat = AudioSystem.getAudioFileFormat(src);
                baseFormat = baseFileFormat.getFormat();
                // TAudioFileFormat properties
                if (baseFileFormat instanceof TAudioFileFormat)
                {
                    Map properties = ((TAudioFileFormat)baseFileFormat).properties();
                    artist = (String) properties.get("author");
                    song_title = (String) properties.get("title");
                    album = (String) properties.get("album");
                    seconds = Integer.parseInt(""+properties.get("duration"))/1000000;
                    //System.out.println(properties.get("duration"));
                    //key = "mp3.id3tag.v2";
                    //InputStream tag= (InputStream) properties.get(key);
                }
                // TAudioFormat properties
                /*
                if (baseFormat instanceof TAudioFormat)
                {
                     Map properties = ((TAudioFormat)baseFormat).properties();
                     String key = "bitrate";
                     Integer val = (Integer) properties.get(key);
                     System.out.println("Bitrate: "+val);
                }*/
            }catch(UnsupportedAudioFileException e)
            {
                e.printStackTrace();
            }
            
            if(seconds < 1)
            {
                //cant read song lenght
                return null;
            }
            
            if(artist==null)
            {
                artist="";
            }
            
            if(album==null || album.equals(""))
            {
                album=toAlbum(path);
            }
            if(song_title==null || song_title.equals(""))
            {
                song_title=toTitel(path);
            }
            
            
            //album = "ingame";
            path = path.replace('\"', ' ');
            song_title = song_title.replace('\"', ' ');
            artist = artist.replace('\"', ' ');
            album = album.replace('\"', ' ');
            
            song = new Song(path, song_title, artist, album, seconds);
   
       
        return song;
    }
    
    public String toTitel(String path) {
        String titel = "";
        try {
            //get title from mp3 file name
            titel = path.substring((path.lastIndexOf('\\') + 1), path.length() - 4);
        } catch (java.lang.StringIndexOutOfBoundsException e) {
            titel = "error";
        }
        return titel;
    }
    
    public String toAlbum(String path) {
        //get subdir
        String album = "";
        try {
            //get path without the file name
            String dirs = path.substring(0,(path.lastIndexOf('\\'))-1);
            //album = path.substring(conf.getAddonPath().length() + 1, path.length() - name.length() - 1);
            album =  dirs.substring(dirs.lastIndexOf('\\')+1,dirs.length());   
        } catch (java.lang.StringIndexOutOfBoundsException e) {
            album = "";
        }
        return album;
    }
    

    private List readPlaylist(InputStream is) {
        
        String line;
        int listnumber = -1;
        ArrayList savedlists = null;
        
        
        try {
            savedlists = new ArrayList();
            BufferedReader in = new BufferedReader(new InputStreamReader(is,"UTF8"));
            
            line = in.readLine();
            int linenumber = 1;
            String line1;
            /** read all*/
            while (line != null) {
                linenumber ++;
                if (line.contains("[\"ListName\"]")) {
                    /** new list found */
                    //getSavedPlaylists().add(new ArrayList());
                    listnumber = listnumber + 1;

                    String[] listname = line.split("\"");
                    //System.out.println(listname[3]);

                    //add list name
                    Song mp3 = new Song("", listname[3], "", "", 0);

                    ArrayList list = new ArrayList();
                    //list = (ArrayList) (getSavedPlaylists().get(listnumber));
                    list.add(mp3);
                    savedlists.add(list);
                    
                } else if (line.contains("[\"Album\"]")) {
                    String[] album = line.split("\"");
                    line = in.readLine();
                    String[] song = line.split("\"");
                    line = in.readLine();
                    String[] name = line.split("\"");
                    line1 = line;
                    //String[] name = line.split("=");
                    //name[1] = name[1].replaceAll(",", "");
                    //name[1] = name[1].replaceAll("\"", "");
                    //name[1] = name[1].trim();
                    line = in.readLine();
                    String[] length = line.split("=");
                    length[1] = length[1].replaceAll(",", "");
                    length[1] = length[1].trim();
                    line = in.readLine();
                    String[] artist = line.split("\"");

                    //System.out.println(song[3]+"  "+name[1]);
                    //System.out.println(album[3] + song[3] + name[3] + artist[3] + Integer.parseInt(length[1]));

                    //Song mp3 = new Song(name[1], song[3], artist[3], album[3], Integer.parseInt(length[1]));
                    
                    try{
                        Song mp3 = new Song(name[3], song[3], artist[3], album[3], Integer.parseInt(length[1]));

                        ArrayList list;
                        list = (ArrayList)savedlists.get(listnumber);
                        list.add(mp3);
                    }
                    catch(IndexOutOfBoundsException e){
                        System.out.println("Error reading linenr "+linenumber+": "+line1);
                        e.printStackTrace();
                    }
                }


                line = in.readLine();
            }
            in.close();
        } catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            savedlists = null;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            savedlists = null;
        }
        catch (ArrayIndexOutOfBoundsException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
            savedlists = null;
        }
        
        if(savedlists.size() < 1){
            return null;
        }
            
            
        return savedlists;

    }
    
    /**
     * reads in existing playlists from savedVariablesPath
     * and save them in list SavedPlaylists 
     * @param filename
     */
    public List readPlaylist(String filename) throws FileNotFoundException {

        InputStream is = new FileInputStream(filename);
        return readPlaylist(is);

    }

     /**
     * parses savedVariablesPath for non playlist information
     * and save it in list savedConfig 
     * @param filename
     */
    public void readSavedVariables(String filename) throws IOException {
        /**
         * check if there ist a saved variable
         */
        File f = new File(filename);
        if(!f.exists())
        {
            return; //no file
        }
        
        
        String line;
        
            BufferedReader in = new BufferedReader(new InputStreamReader
                              (new FileInputStream(filename),"UTF8"));
            line = in.readLine();
            tempCopy.add(line);

            /*
			while(line != null)
			{
				//System.out.println("line: "+line);
				if(line.equals("EpicMusicPlayerDB = {") || line.equals("EpicMusicPlayerDBPC = {") || line.equals("EpicMusicPlayer_RemovedList = {"))
				{
					System.out.println(line);
					while(!line.equals("}") && line != null)
					{
						System.out.println(line);
						this.savedConfig.add(line);
						line = in.readLine();
					}
					this.savedConfig.add(line);
				}
				line = in.readLine();
			}*/

            /** read all*/
            while (line != null) {
                tempCopy.add(line);
                line = in.readLine();
            }
            in.close();

            /** split */
            Iterator it = tempCopy.iterator();
            line = "";
            while (it.hasNext()) {
                line = (String) it.next();
                //System.out.println(line);
                if (line.equals("EpicMusicPlayer_PlayList = nil")) {
                //drop
                } else if (line.equals("EpicMusicPlayer_PlayList = {")) {
                    while (!((String) it.next()).equals("}")) {
                    //drop
                    }
                } /*
				else if (line.equals("EpicMusicPlayer_RemovedList = {"))
				{
					while(!((String)it.next()).equals("}"))
					{
						//drop
					}
				}*/ else {
                    savedConfig.add(line);
                }
            }

        
    }

    /**
     * appends a list of strings to a given file
     * @param list
     * @param filename
     */
    public void writeList(List list, String filename) {
        try {
            //BufferedWriter out = new BufferedWriter(new FileWriter(filename, true));
              BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename,true),"UTF8"));
            
            for (int i = 0; i < list.size(); i++) {
                out.write((String) list.get(i));
                out.newLine();
            }

            out.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * appends a list of arraylists with EpicMusicPlayer objects to a given file
     * @param list
     * @param filename
     */
    public void writePlaylists(List list, String filename, String musicdir) {
        try {
            System.out.println("writing playlist to="+filename);
            
            //BufferedWriter out = new BufferedWriter(new FileWriter(filename, true));
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"UTF8"));
            
            out.newLine();
            out.write("EpicMusicPlayer_PlayList = {");
            out.newLine();

            //write playlists
            for (int listindex = 0; listindex < list.size(); listindex++) {
                
                out.write("  {");
                out.newLine();
                out.write("    {");
                out.newLine();
                ArrayList mp3list;
                mp3list = (ArrayList) (list.get(listindex));
                //write first title of first EpicMusicPlayer in list > playlist name
                out.write("      [\"ListName\"] = \"" + ((Song) mp3list.get(0)).getMp3Title() + "\",");
                out.newLine();
                
                //playlist version
                out.write("      [\"PlaylistVersion\"] = \"" + "1.0" + "\",");
                out.newLine();
                //music dir
                out.write("      [\"MusicDir\"] = \"" +musicdir+ "\",");
                out.newLine();
                
                out.write("    }, -- [" +1+ "]");
                out.newLine();
                
                for (int mp3index = 1; mp3index < mp3list.size(); mp3index++) {    
                    out.write("    {");
                    out.newLine();
                    out.write("      [\"Album\"] = \"" + ((Song) mp3list.get(mp3index)).getSongAlbum() + "\",");
                    out.newLine();
                    out.write("      [\"Song\"] = \"" + ((Song) mp3list.get(mp3index)).getMp3Title() + "\",");
                    out.newLine();
                    out.write("      [\"Name\"] = \"" + ((Song) mp3list.get(mp3index)).getsongPath() + "\",");
                    out.newLine();
                    out.write("      [\"Length\"] = " + ((Song) mp3list.get(mp3index)).getMp3Seconds() + ",");
                    out.newLine();
                    out.write("      [\"Artist\"] =  \"" + ((Song) mp3list.get(mp3index)).getSongArtist() + "\",");
                    out.newLine();
                    out.write("    }, -- [" + (mp3index + 1) + "]");
                    out.newLine();
                }
                out.write("  }, -- [" + (listindex + 1) + "]");
                out.newLine();
            }

            out.write("}");
            out.newLine();
            out.close();
        } catch (IOException e) {
        }
    }

 
    public static int getSeconds(File file) throws BitstreamException, IOException {
        FileInputStream fin;
        fin = new FileInputStream(file);

        BufferedInputStream bin = new BufferedInputStream(fin);
        Bitstream bitstream = new Bitstream(bin);

        Header h = bitstream.readFrame();

        if (h == null) {
            return 0;
        }

        int sek = (int) (h.total_ms(bin.available()) / 1000);
        int min = sek / 60;

        //System.out.println(sek + "   " + min + ":" + sek % 60);

        return sek;
    }

    public static void testing() {
        File dir = new File("C:/Programme/World of Warcraft/Interface/AddOns/EpicMusicPlayer/MP3");
        
        String[] children = dir.list();
        if (children == null) {
        // Either dir does not exist or is not a directory
        } else {
            for (int i = 0; i < children.length; i++) {
                // Get filename of file or directory
                String filename = children[i];
            //System.out.println(filename);
            }
        }

        /*
	    // It is also possible to filter the list of returned files.
	    // This example does not return any files that start with `.'.
	    FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return !name.startsWith(".");
	        }
	    };
	    children = dir.list(filter);
        */


        FilenameFilter mp3filter = new FilenameFilter() {

                    public boolean accept(File dir, String name) {
                        name = name.toUpperCase();
                        //System.out.println(name);
                        return name.endsWith(".MP3");
                    }
                };

        // The list of files can also be retrieved as File objects
        File[] files = dir.listFiles(mp3filter);

        // This filter only returns directories
        FileFilter fileFilter = new FileFilter() {

                    public boolean accept(File file) {
                        return file.isDirectory();
                    }
                };
        File[] dirs = dir.listFiles(fileFilter);

        for (int i = 0; i < dirs.length; i++) {

            //System.out.println(dirs[i]);
            File[] files1 = dirs[i].listFiles();

            //if(files == null) return;
            for (int i1 = 0; i1 < files1.length; i1++) {

                //System.out.println(files1[i1].getName());
            }

        }


    //writePlaylist(files);
    }
    
    /**
     * returns the index of the playlist where given file ist found
     * returns -1 if not found.
     */
    private int checkInPlaylist(String path)
    {
        
        //loop through plalist but skip list 1 the default list
        for (int listindex = 0; listindex < savedPlaylists.size(); listindex++) {
                
                ArrayList mp3list;
                mp3list = (ArrayList) (savedPlaylists.get(listindex));
                
                
                for(int mp3index = 1; mp3index < mp3list.size(); mp3index++)
                {
                    //System.out.println(path+" = "+((EpicMusicPlayer)mp3list.get(mp3index)).getMp3Path());
                    if(path.equals(((Song) mp3list.get(mp3index)).getsongPath()))
                    {
                        //System.out.println(path);
                        return listindex;
                    }
                }
                
                
       }
       return -1;
    }

    public void printSumary(List<String> errors, int notadded, int added, String listname) {
        log("",1);
        log("---------------- Errors, songs not added! ---------------",1);
        
        if(errors.size() < 1){
            log("No errors",1);
        }
        for (String s : errors){
            log(s,0);
        }
        
        log("---------------- Summary ---------------------",1);
        log("",1);
        log(added+" songs added to list \""+listname+"\"",1);
        log(notadded+" songs are already in some playlist.",1);
        log(errors.size()+" errors.",1);
    }

   
    /*
     *  add playlis
     *  return false if listname already exists
     */
    public boolean addList(String listname) {
        this.conf.setSaved(false);
        
        //check if list with listname already exists
        for (int listindex = 0; listindex < savedPlaylists.size(); listindex++) {

            ArrayList mp3list;
            mp3list = (ArrayList) (savedPlaylists.get(listindex));
            //get first title of first EpicMusicPlayer in list > playlist name
            if (listname.equals(((Song) mp3list.get(0)).getMp3Title())) {
                log("Playlist \"" + listname + "\" already exists.",1);
                return false;
            }
        }
        
        //add list
        Song mp3 = new Song("", listname, "", "", 0);
        ArrayList list = new ArrayList();
        list.add(mp3);
        savedPlaylists.add(list);
        return true;
        
    }
    
    
    public void replaceList(ArrayList<Song> l, int listindex)
    {
        savedPlaylists.set(listindex,l);
    }
    
    /*
     *  add files or directory to savedlists at listindex
     */        
    public void addFile(File f, int listindex)
    {
        this.conf.setSaved(false);
        ArrayList summlistinlist = new ArrayList<String>();
        ArrayList summlisterrors = new ArrayList<String>();
        ArrayList mp3list = (ArrayList) (savedPlaylists.get(listindex));
        
        int count = visitAllFiles(f, mp3list,summlistinlist);
        //System.out.println(count);
        //printSumary(SummListErrors,SummListAlreadyInList.size() , count ,getPlaylistName(listindex));
    }
    
    public void clearPlaylist(int listindex)
    {
        ArrayList songlist = (ArrayList) (savedPlaylists.get(listindex));
        Song song = (Song) songlist.get(0); //get playlist name
        songlist.clear();
        songlist.add(song); //add playlist name
    }
    
    public void removeSongs(int songs[], int listindex)
    {
        this.conf.setSaved(false);
        ArrayList mp3list = (ArrayList) (savedPlaylists.get(listindex));
        
        int c=0; //index correction or number of already removed entrys
        for (int i = 0; i < songs.length; i++) {
            
            if(songs[i]>0) //do not remove first entry (playlistname)
            {
                //log("removed: "+((EpicMusicPlayer) mp3list.get(files[i]-c)).getMp3Title());
                mp3list.remove(songs[i]-c);
                //by removig a entry in mp3list the index of the following entrys changes (-1)
                //to match the correct index in files[] with the changed mp3list
                //we need to subtract the number of the already removed entrys 
                //from the stored (old) index in files[i]
                //maybe there ist an easier way to do this^^
                c++;
            }
        }
    }
    
    public void deleteFromDisk(int songs[], int listindex)
    {
        this.conf.setSaved(false);
        ArrayList mp3list = (ArrayList) (savedPlaylists.get(listindex));
        File file;
        Song song;
        
        int c=0; //index correction or number of already removed entrys
        for (int i = 0; i < songs.length; i++) {
            
            if(songs[i]>0) //do not remove first entry (playlistname)
            {
                song = (Song) mp3list.get(songs[i]-c);
                file = new File(conf.getMusicDir()+"/"+song.getsongPath());
                
                if (!file.exists()){
                    log("file does not exist: "+file.getPath(),1);
                }
                else if(!file.canWrite())
                {
                    log("file is wirte protected: "+file.getPath(),1);
                }
                else
                { 
                    if(file.delete())
                    {
                        log("removed from disk: "+file.getPath(),1);
                        mp3list.remove(songs[i]-c);
                        //by removig a entry in mp3list the index of the following entrys changes (-1)
                        //to match the correct index in files[] with the changed mp3list
                        //we need to subtract the number of the already removed entrys 
                        //from the stored (old) index in files[i]
                        //maybe there ist an easier way to do this^^
                        c++;
                    }
                    else
                    {
                        log("error removing file from disk: "+file.getPath(),1);
                    }
                }
            }
        }
    }
    

    /**
     * copy songs from one list to another
     * @param songs: array of songs to move
     * @param listindex: oldlist 
     * @param newlist: newlist
     */
    public void copySongs(int songs[], int listindex, String newlist)
    {
        this.conf.setSaved(false);
        
        //get dest list
        ArrayList<Song> destlist = getPlaylist(getPlaylistIndex(newlist));
        
        ArrayList<Song> srclist = (ArrayList)(savedPlaylists.get(listindex));
        Song s;
        for (int i = 0; i < songs.length; i++) {
            
            if(songs[i]>0) //do not remove first entry (playlistname)
            {
                s = srclist.get(songs[i]);
                System.out.println(s.getMp3Title());
                destlist.add(s);          
            }
        }  
        
    }
    
    /**
     * move songs from one list to another
     * @param songs: array of songs to move
     * @param listindex: oldlist 
     * @param newlist: newlist
     */
    public void moveSongs(int songs[], int listindex, String newlist)
    {
        this.conf.setSaved(false);
        
        //get dest list
        ArrayList<Song> destlist = getPlaylist(getPlaylistIndex(newlist));
        
        ArrayList<Song> srclist = (ArrayList)(savedPlaylists.get(listindex));
        Song s;
        for (int i = 0; i < songs.length; i++) {
            
            if(songs[i]>0) //do not remove first entry (playlistname)
            {
                s = srclist.get(songs[i]);
                System.out.println(s.getMp3Title());
                destlist.add(s);          
            }
        }  
        
        //remove moved songs
        removeSongs(songs, listindex);
    }
    
    
    public int getPlaylistIndex(String listname)
    {
        Song s;
        int i = -1;
        for(ArrayList songlist : savedPlaylists){
            i++;
            s = (Song) songlist.get(0);
            if(s.getMp3Title().equals(listname)){
                return i;
            }    
            
        }
        return i;
    }
    
    
    public void sortPlaylistByPath(int listindex)
    {
        this.conf.setSaved(false);
        if(listindex < savedPlaylists.size())
        {
            ArrayList mp3list;
            mp3list = (ArrayList) (savedPlaylists.get(listindex));
            SongComparator comp = new SongComparator(0); 
            Collections.sort(mp3list,comp);
        }
    }
    
    public void sortPlaylistByTitle(int listindex)
    {
        this.conf.setSaved(false);
        if(listindex < savedPlaylists.size())
        {
            ArrayList mp3list;
            mp3list = (ArrayList) (savedPlaylists.get(listindex));
            SongComparator comp = new SongComparator(1);
            Collections.sort(mp3list,comp);
        }
    }
    
    public void sortPlaylistByArtist(int listindex)
    {
        this.conf.setSaved(false);
        if(listindex < savedPlaylists.size())
        {
            ArrayList mp3list;
            mp3list = (ArrayList) (savedPlaylists.get(listindex));
            SongComparator comp = new SongComparator(2);
            Collections.sort(mp3list,comp);
        }
    }
    
    public void sortPlaylistByAlbum(int listindex)
    {
        this.conf.setSaved(false);
        if(listindex < savedPlaylists.size())
        {
            ArrayList mp3list;
            mp3list = (ArrayList) (savedPlaylists.get(listindex));
            SongComparator comp = new SongComparator(3);
            Collections.sort(mp3list,comp);
        }
    }
    
    public void sortPlaylistByTime(int listindex)
    {
        this.conf.setSaved(false);
        if(listindex < savedPlaylists.size())
        {
            ArrayList mp3list;
            mp3list = (ArrayList) (savedPlaylists.get(listindex));
            SongComparator comp = new SongComparator(4);
            Collections.sort(mp3list,comp);
        }
    }

    public List getNewPlaylist() {
        return newPlaylist;
    }

    public List<ArrayList> getSavedPlaylists() {
        return savedPlaylists;
    }

    public void addStoredIngameList(String listname) {
        //add empty list
        if(!addList(listname))
        {
            return;
        }
        
        //load buildin list
        InputStream is = Playlists.class.getResourceAsStream("/gui/resources/NewList.lua");
        ArrayList lists = (ArrayList) readPlaylist(is);
        
        //get the ingame list
        for (int listindex = 0; listindex < lists.size(); listindex++) {
                
                ArrayList mp3list;
                mp3list = (ArrayList) (lists.get(listindex));
                
                Song s = (Song) mp3list.get(0);
                if(s.getMp3Title().equals("Ingame"))
                {
                    s.setMp3Title(listname);
                    //replace the empty list
                    savedPlaylists.set(savedPlaylists.size()-1, mp3list);
                    return;
                }
         }
        log("Error: Ingame playlist not found.",0);
        
    }
    
    /* todo: move out of this class
     * or even replace with a logger
     */
    public void log(String text, int level)
    {
        Date dt = new Date();
        SimpleDateFormat df = new SimpleDateFormat( "HH:mm:ss" );
        
        if(jLog == null)
        {
            System.out.print("["+df.format(dt)+"] "+text+"\n");
        }
        else
        {
            
            //if(level == 0){
               
                //jPane.a("<html><font color=\"#BB88EE\"> ["+df.format(dt)+"] "+text+" </font></html> \n");
              //  jLog.append("<html><font color=\"#BB88EE\"> ["+df.format(dt)+"] "+text+" </font></html> \n");
            
            //}
            //else {
                //jLog.append("["+df.format(dt)+"] "+text+"\n");
                jLog.append("["+df.format(dt)+"] "+text+"\n");
            //}
            
            
            //jLog.setText( jLog.getText() + "["+df.format(dt)+"] "+text+"\n" );
            jLog.setCaretPosition( jLog.getDocument().getLength() );
            jLog.validate();
        }
    }
}
